import datamart_profiler
import DataProfileViewer
import PipelineProfiler
import VisualTextAnalyzer
from d3m.utils import silence


def plot_metadata(dataset_path):
    with silence():
        metadata = datamart_profiler.process_dataset(dataset_path, plots=True, include_sample=True)

    DataProfileViewer.plot_data_summary(metadata)


def plot_comparison_pipelines(pipelines):
    PipelineProfiler.plot_pipeline_matrix(pipelines)


def plot_text_summary(dataset, text_column, label_column, positive_label, negative_label):
    VisualTextAnalyzer.plot_text_summary(dataset, text_column=text_column, category_column=label_column,
                                         positive_label=positive_label, negative_label=negative_label)



import re
import copy


def is_constructpredictions(step):
    if step['primitive']['python_path'] == 'd3m.primitives.data_transformation.construct_predictions.Common':
        return True
    return False


def is_estimator(step):
    if step['primitive']['python_path'].startswith('d3m.primitives.classification.'):
        return True
    return False


def create_confidence_pipeline(pipeline):
    pipeline = copy.deepcopy(pipeline)
    steps = pipeline['steps']
    new_steps = []
    estimator = None

    for index, step in enumerate(steps):
        if is_estimator(step):
            estimator = step
            estimator['index'] = index
        elif is_constructpredictions(step):
            # Add 3 necessary primitives to calculate confidence
            input_id = int(re.match('steps.(\d+).produce', step['arguments']['inputs']['data']).groups()[0])

            step_horizontal_concat = {
                "type": "PRIMITIVE",
                "primitive": {
                    "id": "aff6a77a-faa0-41c5-9595-de2e7f7c4760",
                    "name": "Concatenate two dataframes",
                    "version": "0.2.0",
                    "python_path": "d3m.primitives.data_transformation.horizontal_concat.DataFrameCommon",
                    "digest": "f1e8fe6ba0456e562d9613bd5f4221e221e9cadd23c684564137b2aa14495ada"
                },
                "arguments": {
                    "left": {
                        "data": estimator['arguments']['inputs']['data'],
                        "type": "CONTAINER"
                    },
                    "right": {
                        "data": estimator['arguments']['outputs']['data'],
                        "type": "CONTAINER"
                    }
                },
                "outputs": [
                    {
                        "id": "produce"
                    }
                ]
            }
            new_steps.append(step_horizontal_concat)

            step_unique_values = {
                "type": "PRIMITIVE",
                "primitive": {
                    "id": "dd580c45-9fbe-493d-ac79-6e9f706a3619",
                    "version": "0.1.0",
                    "name": "Add all_distinct_values to the metadata of the input Dataframe",
                    "python_path": "d3m.primitives.operator.compute_unique_values.Common",
                    "digest": "04178b5c55f24e4f0a2acd5111affe8fc6fe752ab54a380028d439787ec170ef"
                },
                "arguments": {
                    "inputs": {
                        "type": "CONTAINER",
                        "data": 'steps.%d.produce' % (input_id + 1)
                    }
                },
                "outputs": [
                    {
                        "id": "produce"
                    }
                ]
            }
            new_steps.append(step_unique_values)

            step = {
                "type": "PRIMITIVE",
                "primitive": {
                    "id": "500c4f0c-a040-48a5-aa76-d6463ea7ea37",
                    "version": "0.1.0",
                    "name": "Construct confidence",
                    "python_path": "d3m.primitives.data_transformation.construct_confidence.Common",
                    "digest": "8685b43969b06ee2418e2852d8019b4c638813cca5cb2d94962ce782e0ae4651"
                },
                "arguments": {
                    "inputs": {
                        "type": "CONTAINER",
                        "data": 'steps.%d.produce' % (input_id + 2)
                    },
                    "reference": {
                        "type": "CONTAINER",
                        "data": step['arguments']['reference']['data']
                    }
                },
                "outputs": [
                    {
                        "id": "produce"
                    }
                ],
                "hyperparams": {
                    "primitive_learner": {
                        "type": "PRIMITIVE",
                        "data": estimator['index']
                    }
                }
            }

        new_steps.append(step)

    pipeline['steps'] = new_steps
    output_id = int(re.match('steps.(\d+).produce', pipeline['outputs'][0]['data']).groups()[0])
    pipeline['outputs'][0]['data'] = 'steps.%d.produce' % (output_id + 2)

    return pipeline



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='d3m_interface',
    version='0.2.2',
    author='Roque Lopez',
    author_email='rlopez@nyu.edu',
    description='Library to use D3M AutoML Systems',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://gitlab.com/ViDA-NYU/d3m/d3m_interface',
    packages=setuptools.find_packages(),
    license='Apache-2.0',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
    ],
    install_requires=[
        'd3m==2020.11.3',
        'd3m-automl-rpc==1.0.0',
        'pandas==1.0.3',
        'pipelineprofiler==0.1.16',
        'data-profile-viewer==0.2.0',
        'visual-text-explorer==0.1.9',
        'datamart_profiler==0.6.2'
    ],
    python_requires='>=3.6',
    include_package_data=True,
    package_data={'': ['resource/*.json']}
)
